(* Turbo-Pascal-/Delphi-Unit -
   verschiedene Unterroutinen (ohne I/O)
   J. Rathlev, Nov. 1995
      letzte nderung: Dez. 2001 *)


unit TPSub;

interface

type
  DateStr = string[10];
  TCvTable = array [0..127] of char;

const
  ChrID = '#';             (* Kennzeichner fr num. Zeichenwerte *)
  Strich = '-';
  Komma = ',';
  Punkt = '.';
  Semikolon = ';';
  Space = ' ';
  DPunkt = ':';

  DezTr : char = Punkt;    (* Dezimalzeichen *)
  ThMark : char = Space;   (* Zeichen fr Tausendergruppen *)
  TmMark : char = DPunkt;  (* Zeichen fr Uhrzeitgruppen hh:mm:ss *)

  DayInSeconds = 86400;    (* Anzahl der Sekunden am Tag *)

  xDef = 1.0;            (* Standardwert ReadNxtDbl *)
  nDef = 1;              (* Standardwert ReadNxtInt *)
  cDef = #0;             (* Standardwert ReadNxtChr *)

  MName : array[1..12] of string[5] =
      ('Jan.','Febr.','Mrz.','April','Mai','Juni',
       'Juli','Aug.','Sept.','Okt.','Nov.','Dez.');

  MTage : array[1..12] of integer =
      (31,29,31,30,31,30,31,31,30,31,30,31);

  TabToANSI : TCvTable =(
    #$C7,#$FC,#$E9,#$E2,#$E4,#$E0,#$E5,#$E7,#$EA,#$EB,#$E9,#$E8,#$EF,#$EE,#$C4,#$C5,
    #$C9,#$E6,#$C6,#$F4,#$F6,#$F2,#$FB,#$F9,#$FF,#$D6,#$DC,#$A2,#$A3,#$A5,#$20,#$20,
    #$E1,#$ED,#$F3,#$FA,#$F1,#$D1,#$61,#$6F,#$BF,#$20,#$AC,#$BD,#$BC,#$A1,#$AB,#$BB,
    #$20,#$20,#$20,#$A6,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$AD,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$DF,#$B6,#$20,#$20,#$20,#$B5,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$B1,#$20,#$20,#$20,#$20,#$F7,#$20,#$B0,#$20,#$B7,#$20,#$B3,#$B2,#$20,#$20);
  TabToOEM : TCvTable =(
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,#$20,
    #$20,#$AD,#$9B,#$9C,#$20,#$9D,#$B3,#$15,#$20,#$20,#$20,#$AE,#$AA,#$C4,#$20,#$20,
    #$F8,#$F1,#$FD,#$FC,#$20,#$E6,#$E3,#$20,#$20,#$20,#$20,#$AF,#$AC,#$AB,#$20,#$A8,
    #$41,#$41,#$41,#$41,#$8E,#$8F,#$92,#$80,#$45,#$90,#$45,#$45,#$49,#$49,#$49,#$49,
    #$44,#$A5,#$4F,#$4F,#$4F,#$4F,#$99,#$20,#$20,#$55,#$55,#$55,#$9A,#$59,#$20,#$E1,
    #$85,#$61,#$83,#$61,#$84,#$86,#$91,#$87,#$8A,#$82,#$88,#$89,#$8D,#$A1,#$8C,#$8B,
    #$20,#$A4,#$95,#$A2,#$93,#$6F,#$94,#$F6,#$20,#$97,#$A3,#$96,#$81,#$79,#$20,#$98);

  type
    TWinkelFormat = (fmDezGrad,fmDezMin,fmSekunden);

    TLongWord = record
      case integer of
      0 : (LongWord : longint);
      1 : (Lo,Hi : word);
      2 : (LoL,LoH,HiL,HiH : byte);
      end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines DOS-Zeichens in eines Grobuchstaben (auch Umlaute) *)
function OEMUpCaseG (c : char) : char;

(* Umwandeln eines DOS-Strings in Grobuchstaben (auch Umlaute) *)
function OEMUpString (S : string) : string;

(* Umwandeln eines ANSI-Strings in Grobuchstaben (auch Umlaute) *)
function ANSIUpString (S : string) : string;

(* Umwandeln eines Strings in Grobuchstaben (Umlaute in AE, OE, UE)
   zu Sortierzwecken *)
function OEMUpSort (S : string) : string;

(* Umwandeln eines OEM-Strings nach ANSI *)
function StrToAnsi (s : string) : string;

(* Umwandeln eines ANSI-Strings nach OEM *)
function StrToOEM (s : string) : string;

(* Integer-Zahl in String in Dreiergruppen umsetzen*)
function NStrInt (x : longint;
                  n : integer) : string;

(* Zeichen (Old) in einem String durch ein anderes (New) ersetzen *)
procedure ReplChar (var s   : openstring;
                    Old,New : char);
{ ---------------------------------------------------------------- }
(* String mit Leerstellen erzeugen *)
function FillSpace (len : integer) : string;

(* fhrende Leerstellen ergnzen *)
function AddSp (S   : string;
                len : integer) : string;

(* Leerstellen am Ende ergnzen *)
function ExtSp (S   : string;
                len : integer) : string;

(* fhrende Leerstellen lschen *)
function RemSp (S : string) : string;

(* Leerstellen am Ende lschen *)
function CutSp (S : string) : string;

(* alle Leerstellen im String lschen *)
function DelSp (S : string) : string;

(* erstes nichtleeres Zeichen suchen *)
function SeekNoSp (s : string) : byte;

(* String mit Zeichen erzeugen *)
function FillStr (c   : char;
                  len : integer) : string;

(* fhrende Zeichen ergnzen *)
function AddChar (S   : string;
                  c   : char;
                  len : integer) : string;

(* Zeichen am Ende ergnzen *)
function ExtChar (S   : string;
                  c   : char;
                  len : integer) : string;

(* String auf feste Lnge bringen (krzen oder mit Leerzeichen auffllen) *)
function FixLength (S   : string;
                    len : integer) : string;

{ ---------------------------------------------------------------- }
(* Ausmaskieren der Erweiterung *)
function GetExt (name : string) : string;

(* Dateinamenerweiterung entfernen *)
function DelExt (Name : string) : string;

(* ndern einer Dateinamenerweiterung *)
function NewExt (Name,Ext  : string) : string;

(* Dateiname mit Laufwerk, Verzeichnis und
   Extension (falls nicht vorhanden) erweitern *)
function Erweiter (Pfad,Name,Ext : string) : string;

(* Pfad aus Dateinamen teilweise entfernen, wenn der Name zu lang wird *)
function StripPath (Name : String;
                    Len  : integer) : String;

(* Bei Verzeichnisangaben Backslash ergnzen *)
function SetDir (Dir : string) : string;

{ ---------------------------------------------------------------- }
(* einen Zeitstring (hh[:mm[:ss]]) in die Tageszeit in Sekunden umrechnen *)
function TPStrToTime (S         : String;
                      var error : boolean) : longint;

(* Tageszeit in s nach Stunden:Minuten[:Sekunden] umrechnen *)
function TPTimeToStr (Time : longint;
                      n,m  : integer) : string;

(* aus Datumstring (dd.mm.(jj)jj) die Anzahl der Tage seit 1950 ermitteln *)
procedure DateToDays (Date     : DateStr;
                      var Days : word;
                      var Err  : boolean);

(* aus der Anzahl der Tage seit 1950 das Datum (als String) ermitteln
   1950 <= Jahreszahl < 2100 *)
function DaysToDate (Days : word;
                     Trn  : char) : DateStr;      (* Trennzeichen *)

(* Datum in Anzahl der Tage seit 1900 umrechnen *)
function DateToDaysFrom1900 (Tag,Monat,Jahr : integer) : word;

(* Datum aus Anzahl der Tage seit 1900 berechnen *)
procedure DaysFrom1900ToDate (Days               : word;
                              var Tag,Monat,Jahr : integer);

{ ---------------------------------------------------------------- }
(* String in Winkel konvertieren *)
function StrToDeg (s     : string;
                   var v : double) : boolean;

(* Winkel in String mit vorgebenem Format konvertieren *)
function DegToStr(v      : double;
                  Format : TWinkelFormat;
                  w,d    : integer) : string;

{ ---------------------------------------------------------------- }
(* String bis zum nchsten Leerzeichen extrahieren,
   fhrende Leerzeichen werden ignoriert,
   der String s wird entsprechend gekrzt *)
function GetNumStr (var s : string) : string;

{ ---------------------------------------------------------------- }
(* Integer, Double oder String aus einem String s bis zum nchsten
   Begrenzer (Del) lesen
   s wird um den verarbeiteten Teil gekrzt *)
function ReadNxtInt (var s   : OpenString;
                     Del     : char;
                     Default : integer;
                     var err : boolean) : integer;

function ReadNxtLongInt (var s   : OpenString;
                         Del     : char;
                         Default : longint;
                         var err : boolean) : longint;

(*$IFOPT N+ *)
function ReadNxtDbl (var s   : OpenString;
                     Del     : char;
                     Default : double;
                     var err : boolean) : double;
(*$ENDIF *)

function ReadNxtStr (var s   : OpenString;
                     Del     : char) : string;

function ReadNxtChr (var s   : OpenString;
                     Del     : char;
                     Default : char;
                     var err : boolean) : char;

(* Zeit in Sek. aus String (hh:mm:ss) extrahieren *)
function ReadNxtTime (var s   : OpenString;
                      Del     : char;
                      Default : longint;
                      var err : boolean) : longint;

{ ---------------------------------------------------------------- }
(* Testen, ob gltige Integer oder Double Zahl in einem String s
   bis zum nchsten Begrenzer (Del) steht *)
function TestNxtNumber (s       : OpenString;
                        Del     : char ) : boolean;

{ ---------------------------------------------------------------- }
(* Dezimaltrenner setzen und lesen, Standard = Punkt *)
procedure SetDecPtChar (Zeichen : char);
function GetDecPtChar : char;

(* Tausendermarkierung, Standard = Leerzeichen *)
procedure ThMarkChar (Zeichen : char);

(* Integer-Zahl in String umsetzen *)
function StrInt (x : longint;
                 n : integer) : string;

(*$IFOPT N+ *)
(* Double-Zahl in String umsetzen *)
function StrDbl (x   : double;
                 n,d : integer) : string;
(*$ENDIF *)

(* Integer-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrInt (x : longint;
                  n : integer) : string;

(*$IFOPT N+ *)
(* Double-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrDbl (x   : double;
                  n,d : integer) : string;

(* Double-Zahl in String in Dreiergruppen umsetzen*)
function NStrDbl (x : double;
                  n,d : integer) : string;
(*$ENDIF *)

(* Konvertierung einer 16-bit oder 8-Bit Zahl in einen String,
   der dem Hex-Wert entspricht *)
function HexByte (b : byte) : string;

function HexWord (w : word) : string;

{ ---------------------------------------------------------------- }
(*$IFOPT N+ *)
(* Wert auf Gltigkeit prfen
   true : x <> NoVal *)
function Valid (x,NoVal : double) : boolean;
(*$ENDIF *)

{ ---------------------------------------------------------------- }
(* Dezimalkomma ersetzen *)
function ReplDecPt (s : string) : string;

(* Dezimalpunkt einsetzen *)
function SetDecPt (s : string) : string;

{ ---------------------------------------------------------------- }
(* Longinteger in Hi- oder Lo-Wort zerlegen *)
function HiWord (n : longint) : word;

function LoWord (n : longint) : word;

(* Longinteger aus Hi- und Lo-Wort zusammensetzen *)
function JoinWord (HiWord,LoWord : word) : longint;

(* Longinteger aus 4 Bytes zusammensetzen *)
function JoinByte (HHByte,HLByte,LHByte,LLByte : word) : longint;

implementation
{ ---------------------------------------------------------------- }
(* Umwandeln eines Zeichens in eines Grobuchstaben (auch Umlaute)
   OEM-Zeichen *)
function OEMUpCaseG (c : char) : char;
begin
  if c<#127 then OEMUpCaseG:=upcase(c)
  else begin
    case c of
    '' : OEMUpCaseG:='';
    '' : OEMUpCaseG:='';
    '' : OEMUpCaseG:='';
    '' : OEMUpCaseG:='S';
      else OEMUpCaseG:=c;
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Strings in Grobuchstaben (auch Umlaute) *)
function OEMUpString (S : string) : string;
var
  i  : integer;
  SL : byte absolute s;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to SL do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  OEMUpString:=ns;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines ANSI-Strings in Grobuchstaben (auch Umlaute) *)
function ANSIUpString (S : string) : string;
var
  i  : integer;
  SL : byte absolute s;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to SL do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'';
      '' : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  ANSIUpString:=ns;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines Strings in Grobuchstaben (Umlaute in AE, OE, UE)
   zu Sortierzwecken *)
function OEMUpSort (S : string) : string;
var
  i  : integer;
  SL : byte absolute s;
  ns : string;
  c  : char;
begin
  ns:='';
  for i:=1 to SL do begin
    c:=s[i];
    if c<#127 then ns:=ns+upcase(c)
    else begin
      case c of
      '','' : ns:=ns+'AE';
      '','' : ns:=ns+'OE';
      '','' : ns:=ns+'UE';
      ''     : ns:=ns+'SS';
        else ns:=ns+c;
        end;
      end;
    end;
  OEMUpSort:=ns;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines OEM-Strings nach ANSI *)
function StrToAnsi (s : string) : string;
var
  i  : integer;
  SL : byte absolute s;
begin
  for i:=1 to SL do begin
    if s[i]>=#128 then s[i]:=TabToANSI[ord(s[i])-128]
    else if s[i]=#$15 then s[i]:=#$A7;
  {   case s[i] of
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    #$F8 : s[i]:='';
      end;}
    end;
  StrToAnsi:=s;
  end;

{ ---------------------------------------------------------------- }
(* Umwandeln eines ANSI-Strings nach OEM *)
function StrToOEM (s : string) : string;
var
  i  : integer;
  SL : byte absolute s;
begin
  for i:=1 to SL do begin
    if s[i]>=#128 then s[i]:=TabToOEM[ord(s[i])-128];
{    case s[i] of
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:='';
    '' : s[i]:=#$F8;
      end; }
    end;
  StrToOEM:=s;
  end;

{------------------------------------------------------------------}
(* Zeichen (Old) in einem String durch ein anderes (New) ersetzen *)
procedure ReplChar (var s   : openstring;
                    Old,New : char);
var
  sl : byte absolute s;
  i  : integer;
begin
  for i:=1 to sl do if s[i]=Old then s[i]:=New;
  end;

{------------------------------------------------------------------}
(* String mit Leerstellen erzeugen *)
function FillSpace (len : integer) : string;
var
  i : byte;
  s : string;
begin
  if len>255 then len:=255;
  for i:=1 to len do s[i]:=' ';
  s[0]:=char(len);
  FillSpace:=s;
  end;

{------------------------------------------------------------------}
(* fhrende Leerstellen ergnzen *)
function AddSp (S   : string;
                len : integer) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  for i:=succ(sl) to len do S:=' '+S;
  AddSp:=S;
  end;

{------------------------------------------------------------------}
(* Leerstellen am Ende ergnzen *)
function ExtSp (S   : string;
                len : integer) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  for i:=succ(sl) to len do S:=S+' ';
  ExtSp:=S;
  end;

{------------------------------------------------------------------}
(* fhrende Leerstellen lschen *)
function RemSp (S : string) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  i:=1;
  while (i<=sl)  and (S[i]=#32) do inc(i);
  delete(S,1,pred(i));
  RemSp:=s;
  end;

{ --------------------------------------------------------------- }
(* Leerstellen am Ende lschen *)
function CutSp (S : string) : string;
var
  sl : byte absolute S;
begin
  while (sl>0) and (s[sl]=' ') do dec(sl);
  CutSp:=s;
  end;

{ --------------------------------------------------------------- }
(* alle Leerstellen im String lschen *)
function DelSp (S : string) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  i:=1;
  while (i<=sl) do begin
    if S[i]=#32 then delete(S,i,1)
    else inc(i);
    end;
  DelSp:=s;
  end;

{ --------------------------------------------------------------- }
(* erstes nichtleeres Zeichen suchen *)
function SeekNoSp (s : string) : byte;
var
  i  : byte;
  sl : byte absolute s;
begin
  i:=1;
  while (i<=sl) and (s[i]=Space) do inc(i);
  if i<=sl then SeekNoSp:=i else SeekNoSp:=0;
  end;

{ --------------------------------------------------------------- }
(* String mit Zeichen erzeugen *)
function FillStr (c   : char;
                  len : integer) : string;
var
  i : byte;
  s : string;
begin
  if len>255 then len:=255;
  for i:=1 to len do s[i]:=c;
  s[0]:=char(len);
  FillStr:=s;
  end;

{ --------------------------------------------------------------- }
(* fhrende Zeichen ergnzen *)
function AddChar (S   : string;
                  c   : char;
                  len : integer) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  for i:=succ(sl) to len do S:=c+S;
  AddChar:=S;
  end;

{ --------------------------------------------------------------- }
(* Zeichen am Ende ergnzen *)
function ExtChar (S   : string;
                  c   : char;
                  len : integer) : string;
var
  sl : byte absolute S;
  i  : integer;
begin
  for i:=succ(sl) to len do S:=S+c;
  ExtChar:=S;
  end;

{ --------------------------------------------------------------- }
(* String auf feste Lnge bringen (krzen oder mit Leerzeichen auffllen *)
function FixLength (S   : string;
                    len : integer) : string;
var
  sl : byte absolute S;
begin
  if sl<len then FixLength:=ExtSp (S,len)
  else FixLength:=copy (s,1,len);
  end;

{ --------------------------------------------------------------- }
(* Ausmaskieren der Erweiterung *)
function GetExt (name : string) : string;
var
  i : integer;
begin
  i:=pos('.',name);
  if i=0 then GetExt:=''
  else GetExt:=copy(name,succ(i),3);
  end;

{ --------------------------------------------------------------- }
(* Dateinamenerweiterung entfernen *)
function DelExt (Name : string) : string;
var
  i : integer;
begin
  i:=pos ('.',Name); Name:=OEMUpString(Name);
  if i<>0 then DelExt:=copy(Name,1,pred(i)) else DelExt:=Name;
  end;

{ --------------------------------------------------------------- }
(* ndern einer Dateinamenerweiterung *)
function NewExt (Name,Ext  : string) : string;
var
  i : integer;
begin
  name:=OEMUpString (name); i:=pos('.',name);
  if i<>0 then name:=copy (name,1,i)
  else name:=name+'.';
  if Ext[1]='.' then delete(Ext,1,1);
  if length(ext)<>0 then NewExt:=name+ext
  else NewExt:=copy(name,1,pred(length(name)));
  end;

{ --------------------------------------------------------------- }
(* Dateiname mit Laufwerk, Verzeichnis und
   Extension (falls nicht vorhanden) erweitern *)
function Erweiter (Pfad,Name,Ext  : string) : string;
var
  stemp : string;
begin
  if Ext[1]='.' then delete(Ext,1,1);
  if (pos('.',Name)=0) and (length(Ext)<>0) then Name:=Name+'.'+ext;
  if (pos (':',Name)<>0) or (pos('\',Name)<>0)
     or (length(Pfad)=0) then stemp:=Name
  else if Pfad[length(Pfad)]='\' then stemp:=Pfad+Name
         else stemp:=Pfad+'\'+Name;
  erweiter:=OEMUpString(stemp);
  end;

{ ---------------------------------------------------------------- }
(* Pfad aus Dateinamen teilweise entfernen, wenn der Name zu lang wird *)
function StripPath (Name : String;
                    Len  : integer) : String;
const
  Punkte = '...';
var
  i,j : integer;
  nl  : byte absolute Name;
  ok  : boolean;
begin
  if nl>=Len then begin
    i:=nl; ok:=true;
    while (Name[i]<>'\') and (i>0) do dec(i);
    if i=0 then Name:=Punkte+copy(Name,nl-Len+3,nl)
    else begin
      dec(i); j:=i;
      repeat
        while (Name[j]<>'\') and (j>0) do dec(j);
        dec(j);
        if j<0 then Name:=Punkte+copy(Name,nl-Len+3,nl)
        else begin
          ok:=nl-i+j+4<=Len;
          end;
        until ok or (j<0);
      if ok then begin
        inc(j,2);
        delete (Name,j,succ(i-j)); insert ('...',Name,j);
        end;
      end;
    end;
  StripPath:=Name;
  end;

{ ------------------------------------------------------------------- }
(* Bei Verzeichnisangaben Backslash ergnzen *)
function SetDir (Dir : string) : string;
var
  dl : byte absolute Dir;
begin
  if (length(Dir)=0) or (Dir[dl]='\') then SetDir:=Dir
  else SetDir:=Dir+'\';
  end;

{ ------------------------------------------------------------------- }
(* einen Zeitstring (hh[:mm[:ss]]) in die Tageszeit in Sekunden
   umrechnen
   Trennzeichen: siehe globale Var. "TmMark" *)
function TPStrToTime (S         : String;
                      var error : boolean) : longint;
var
  sl         : byte absolute S;
  xs         : longInt;
  xm,xk,i,ic : integer;
begin
  i:=1; error:=false; xm:=0; xk:=0;
  while (S[i]=#32) and (i<sl) do inc(i); delete (S,1,pred(i));
  i:=pos(TmMark,S);
  if i=0 then i:=pos(#32,S);
  if i=0 then i:=succ(sl);
  if sl>0 then begin
    val(copy(S,1,pred(i)),xs,ic);  (* Stunden *)
    if (ic<>0) or (xs<0) then error:=true
    else begin
      S:=copy(S,succ(i),sl);
      while (sl>0) and (S[1]=#32) do delete(s,1,1);
      if sl>0 then begin
        i:=pos(TmMark,S);
        if i=0 then i:=pos(#32,S);
        if i=0 then i:=succ(sl);
        val(copy(S,1,pred(i)),xm,ic);   (* Minuten *)
        if (ic<>0) or (xm>=60) or (xm<0) then error:=true
        else begin
          S:=copy(S,succ(i),sl);
          while (sl>0) and (S[1]=#32) do delete(s,1,1);
          if sl>0 then begin
            val(S,xk,ic);       (* Sekunden *)
            if (ic<>0) or (xk>=60) or (xk<0) then error:=true;
            end;
          end;
        end;
      end;
    end
  else error:=true;
  if not error then begin
    xs:=((60*xs+xm)*60+xk);
    if xs>DayInSeconds then xs:=DayInSeconds;
    TPStrToTime:=xs;
    end
  else TPStrToTime:=0;
  end;

{ ------------------------------------------------------------------- }
(* Tageszeit in s nach Stunden:Minuten[:Sekunden] umrechnen
    String mit Leerzeichen auffllen bis zur Lnge  *)
function TPTimeToStr (Time : longint;
                      n,m  : integer) : string;
(* Time : Uhrzeit in s
   n    : Anzahl der Stellen (am Ende wird mit Leerzeichen aufgefllt)
   m    : Modus = 0 : Stunden:Minuten(:Sekunden, wenn <> 0)
                  1 : Stunden:Minuten
                  2 : Stunden:Minuten:Sekunden *)
var
  s            : String;
  sl           : byte absolute s;
  std,min,sek : integer;
begin
  sek:=Time mod 60; Time:=Time div 60;
  min:=Time mod 60; std:=Time div 60;
  s:=ZStrInt(std,2)+TmMark+ZStrInt(min,2);
  if (m>1) or ((m=0) and (sek>0)) then s:=s+TmMark+ZStrInt(sek,2);
  if n>sl then for sek:=succ(sl) to n do s:=s+' ';
  TPTimeToStr:=s;
  end;

{ ------------------------------------------------------------------- }
(* aus einem Datumstring (dd.mm.[jj]jj) die Anzahl der Tage
   seit 1950 ermitteln (Monat als Zahl)
   entweder: (19)50 <= jj < (20)49
   oder 1950 <= jjjj *)
const
  D1950 = 18325;  (* Tage seit 1900 *)
procedure DateToDays (Date     : DateStr;
                      var Days : word;
                      var Err  : boolean);
var
  Tag,Monat,Jahr,ic,i : integer;
begin
  err:=false;
  i:=pos('.',Date);
  val (copy(Date,1,pred(i)),Tag,ic);
  err:=err or (ic<>0) or (Tag<1);
  delete(Date,1,i); i:=pos('.',Date);
  val (copy(Date,1,pred(i)),Monat,ic);
  err:=err or (ic<>0) or (Monat<1) or (Monat>12) or (Tag>MTage[Monat]);
  delete (Date,1,i);
  val (copy(Date,1,4),Jahr,ic); err:=err or (ic<>0);
  err:=err or ((Monat=2) and (Jahr mod 4<>0) and (Tag>28));
  if Jahr<50 then inc(Jahr,100);
  if Jahr>=1950 then dec (Jahr,1900);
  err:=err or (Jahr<0);
  if err then Days:=0
  else begin
    if Monat<=2 then begin
      inc(Monat,12); dec (Jahr,1);
      end;
    Days:=Trunc(Tag+Int(30.6001*succ(Monat))+Int(365.25*Jahr))-D1950;
    end;
  end;

{ ------------------------------------------------------------------- }
(* aus der Anzahl der Tage seit 1950 das Datum (als String) ermitteln
   1950 <= Jahreszahl < 2100 *)
function DaysToDate (Days : word;
                     Trn  : char) : DateStr;      (* Trennzeichen *)
var
  Tag,Monat,Jahr,DayOfWeek : integer;
begin
  inc(Days,D1950);
  DayOfWeek:=Days mod 7;  (* Wochentag: Sonntag = 0 (fr Erweiterungen) *)
  Jahr:=trunc((Days-122.1)/365.25); dec (Days,trunc(Jahr*365.25));
  Monat:=trunc(Days/30.6001);
  Tag:=Days-trunc(Monat*30.6001); dec(Monat);
  if Monat>12 then dec(Monat,12);
  if Monat<=2 then inc(Jahr);
  DaysToDate:=StrInt(Tag,2)+Trn+StrInt(Monat,2)+Trn+StrInt(1900+Jahr,4);
  end;

{ ------------------------------------------------------------------- }
(* Datum in Anzahl der Tage seit 1900 umrechnen *)
function DateToDaysFrom1900 (Tag,Monat,Jahr : integer) : word;
begin
  if Jahr<50 then inc(Jahr,100);
  if Jahr>=1950 then dec (Jahr,1900);
  if Monat<=2 then begin
    inc(Monat,12); dec (Jahr,1);
    end;
  DateToDaysFrom1900:=Trunc(Tag+Int(30.6001*succ(Monat))+Int(365.25*Jahr));
  end;

{ ------------------------------------------------------------------- }
(* Datum aus Anzahl der Tage seit 1900 berechnen *)
procedure DaysFrom1900ToDate (Days               : word;
                              var Tag,Monat,Jahr : integer);
begin
  Jahr:=trunc((Days-122.1)/365.25); dec (Days,trunc(Jahr*365.25));
  Monat:=trunc(Days/30.6001);
  Tag:=Days-trunc(Monat*30.6001); dec(Monat);
  if Monat>12 then dec(Monat,12);
  if Monat<=2 then inc(Jahr);
  inc(Jahr,1900);
  end;

{------------------------------------------------------------------}
(* Winkel-Wert konvertieren *)
function StrToDeg (s     : string;
                   var v : double) : boolean;
var
 t      : String;
 x,y    : double;
 i,j,k  : integer;
begin
  j:=0;
  k:=pos(',',s); if k>0 then s[k]:='.';
  if length(s)<>0 then begin
    k:=pos(':',s);
    if k=0 then k:=pos('/',s);
    if k=0 then k:=pos('"',s);
    if k=0 then val(s,v,j)
    else begin (* Grad:Min *)
      val (copy(s,1,pred(k)),x,j); (* Grad *)
      if j=0 then begin
        t:=copy (s,succ(k),255);
        k:=pos(':',t);
        if k=0 then k:=pos('/',t);
        if k=0 then k:=pos(#$27,s);
        if k=0 then begin (* dez.Min *)
          if length(t)>0 then begin
            val (t,v,j);
            if (j=0) and (v<60.0) then begin
              if x>=0.0 then v:=x+v/60.0
              else v:=x-v/60.0;
              end
            else j:=1;
            end
          else v:=x;
          end
        else begin (* Sek. *)
          val (copy (t,1,pred(k)),y,j);  (* Min *)
          if (j=0) and (y<60.0) then begin
            t:=copy (t,succ(k),255);
            if length(t)>0 then begin
              val (t,v,j); (* Sek. *)
              if (j=0) and (v<60.0) then begin
                if x>=0.0 then v:=x+(y+v/60.0)/60.0
                else v:=x-(y-v/60.0)/60.0;
                end
              else j:=1;
              end
            else begin
              if x>=0.0 then v:=x+y/60.0
              else v:=x-y/60.0;
              end;
            end
          else j:=1;
          end;
        end;
      end;
    end
  else v:=0.0;
  StrToDeg:=j=0;
  end;

{------------------------------------------------------------------}
(* Winkel in String mit vorgebenem Format umsetzen
   Stellen: w : gesamt
            d : hinter Komma (einschl. Minuten und Sekunden
   Typ : m = fmDezGrad  : Grad,xxx
             fmDezMin   : Grad:Minuten,xx
             fmSekunden : Grad:Minuten:Sekunden,xxx *)
function DegToStr(v      : double;
                  Format : TWinkelFormat;
                  w,d    : integer) : string;
var
  s,t   : String;
  x,y,z : extended;
  i     : integer;
begin
  x:=int(abs(v)); y:=60.0*frac(abs(v));
  case Format of
  fmDezGrad : str(v:w:d,s);
  fmDezMin : begin (* Grad:Min *)
      str (y:(w-6):(d-2),t);
      if copy(t,1,2)='60' then begin
        y:=0.0; x:=x+1.0;
        str (y:(w-6):(d-2),t);
        end;
      if v<0.0 then x:=-x;
      str(x:5:0,s); s:=s+':'+t;
      end;
  fmSekunden : begin (* Grad:Min:Sek *)
      z:=int(y); y:=60.0*frac(y);
      str(y:(w-8):(d-4),t);
      if copy(t,1,2)='60' then begin
        y:=0.0; z:=z+1.0;
        if z>=60.0 then begin
          z:=0.0; x:=x+1.0;
          end;
        str (y:(w-8):(d-4),t);
        end;
      str (z:2:0,s); t:=s+':'+t;
      if v<0.0 then x:=-x;
      str(x:4:0,s); s:=s+':'+t;
      end;
    end;
  i:=pos('.',s); if i>0 then s[i]:=',';
  DegToStr:=s;
  end;

{ ------------------------------------------------------------------- }
(* String bis zum nchsten Leerzeichen extrahieren,
   fhrende Leerzeichen werden ignoriert,
   der String s wird entsprechend gekrzt *)
function GetNumStr (var s : string) : string;
var
  sl  : byte absolute s;
  i,j : integer;
  t   : string;
begin
  i:=1;
  while (i<=sl) and (s[i]=Space) do inc(i);
  if i>sl then begin
    t:=''; s:='';
    end
  else begin
    j:=i;
    while (j<=sl) and (s[j]<>Space) do inc(j);
    t:=copy(s,i,j-i);
    delete (s,1,j);
    end;
  GetNumStr:=t;
  end;

{ ------------------------------------------------------------------- }
(* Integer, Double oder String aus einem String s bis zum nchsten
   Komma lesen
   s wird um den verarbeiteten Teil gekrzt *)
function ReadNxtInt (var s   : OpenString;
                     Del     : char;
                     Default : integer;
                     var err : boolean) : integer;
var
  sl     : byte absolute s;
  n,i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(sl);
  val(copy(s,1,pred(i)),n,ic);
  if ic=0 then ReadNxtInt:=n
  else begin
    ReadNxtInt:=Default;
    err:=true;
    end;
  delete(s,1,i);
  end;

{ ------------------------------------------------------------------- }
function ReadNxtLongInt (var s   : OpenString;
                         Del     : char;
                         Default : longint;
                         var err : boolean) : longint;
var
  sl   : byte absolute s;
  n    : longint;
  i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(sl);
  val(copy(s,1,pred(i)),n,ic);
  if ic=0 then ReadNxtLongInt:=n
  else begin
    ReadNxtLongInt:=Default;
    err:=true;
    end;
  delete(s,1,i);
  end;

{ ------------------------------------------------------------------- }
(*$IFOPT N+ *)
function ReadNxtDbl (var s   : OpenString;
                     Del     : char;
                     Default : double;
                     var err : boolean) : double;
var
  sl   : byte absolute s;
  i,ic : integer;
  x    : double;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(sl);
  val(SetDecPt(copy(s,1,pred(i))),x,ic);
  if ic=0 then ReadNxtDbl:=x
  else begin
    ReadNxtDbl:=Default;
    err:=true;
    end;
  delete(s,1,i);
  end;
(*$ENDIF *)

{ ------------------------------------------------------------------- }
function ReadNxtStr (var s   : OpenString;
                     Del     : char) : string;
var
  sl   : byte absolute s;
  i,ic : integer;
begin
  if length(s)>0 then begin
    i:=pos (Del,s);
    if i=0 then i:=succ(sl);
    ReadNxtStr:=copy(s,1,pred(i));
    delete(s,1,i);
    end
  else ReadNxtStr:='';
  end;

{ ------------------------------------------------------------------- }
(* ein Zeichen lesen oder num. Wert #nnn in Zeichen umwandeln *)
function ReadNxtChr (var s   : OpenString;
                     Del     : char;
                     Default : char;
                     var err : boolean) : char;
var
  sl     : byte absolute s;
  n,i,ic : integer;
begin
  i:=pos (Del,s);
  if s[1]=ChrID then begin
    if i=0 then i:=succ(sl);
    val(copy(s,2,pred(i)),n,ic);
    if ic=0 then ReadNxtChr:=chr(n)
    else begin
      ReadNxtChr:=Default; err:=true;
      end;
    end
  else ReadNxtChr:=s[1];
  delete(s,1,i);
  end;

{ ------------------------------------------------------------------- }
(* Testen, ob gltige Integer oder Double Zahl in einem String s
   bis zum nchsten Begrenzer (Del) steht *)
function TestNxtNumber (s       : OpenString;
                        Del     : char ) : boolean;
var
  sl     : byte absolute s;
  n,i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(sl);
  val(copy(s,1,pred(i)),n,ic);
  TestNxtNumber:=ic=0;
  end;

{ ------------------------------------------------------------------- }
(* Zeit in Sek. aus String (hh:mm:ss) extrahieren *)
function ReadNxtTime (var s   : OpenString;
                      Del     : char;
                      Default : longint;
                      var err : boolean) : longint;
var
  tm        : longint;
  ierr      : boolean;
  st        : string;
  c         : char;
begin
  ierr:=false;
  st:=ReadNxtStr (s,Del);
  tm:=TPStrToTime (st,ierr);
  err:=err or ierr;
  if ierr then ReadNxtTime:=Default else ReadNxtTime:=tm;
  end;

{ ---------------------------------------------------------------- }
(* Dezimaltrenner auf Zeichen setzen, Standard = Punkt *)
procedure SetDecPtChar (Zeichen : char);
begin
  DezTr:=Zeichen;
  end;

{ ---------------------------------------------------------------- }
(* aktuellen Dezimaltrenner lesen *)
function GetDecPtChar : char;
begin
  GetDecPtChar:=DezTr;
  end;

{ ---------------------------------------------------------------- }
(* Tausendermarkierung, Standard = Leerzeichen *)
procedure ThMarkChar (Zeichen : char);
begin
  ThMark:=Zeichen;
  end;

{ ---------------------------------------------------------------- }
(* Integer-Zahl in String umsetzen *)
function StrInt (x : longint;
                 n : integer) : string;
(* n > 0 : Anzahl der Stellen
     = 0 : unformatiert
     < 0 : unformatiert mit Dreiergruppierung *)
var
  s   : string;
  i,j : integer;
begin
  if n<=0 then str(x,s) else str (x:n,s);
  if n<0 then begin
    i:=succ(length(s));
    j:=i;
    while (j>4) do begin
      dec(j,3); insert (ThMark,s,j);
      end;
    end;
  StrInt:=s;
  end;

{ ---------------------------------------------------------------- }
(*$IFOPT N+ *)
(* Double-Zahl in String umsetzen *)
function StrDbl (x   : double;
                 n,d : integer) : string;
var
  s : string;
begin
  str (x:n:d,s);
  if DezTr<>Punkt then s:=ReplDecPt (s);
  StrDbl:=s;
  end;
(*$ENDIF *)

{ ---------------------------------------------------------------- }
(* Integer-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrInt (x : longint;
                  n : integer) : string;
var
  s : string;
  i : integer;
begin
  str (abs(x):n,s);
  if x<0 then if s[1]=Space then s[1]:='-' else s:='-'+s;
  for i:=1 to n do if s[i]=Space then s[i]:='0';
  ZStrInt:=s;
  end;

{ ------------------------------------------------------------------ }
(* Integer-Zahl in String in Dreiergruppen umsetzen*)
function NStrInt (x : longint;
                  n : integer) : string;
(* Format wie StrInt *)
var
  s : string;
  i : integer;
begin
  s:=StrInt(x,n);
  i:=succ(length(s));
  while (i>4) do begin
    dec(i,3); insert (ThMark,s,i);
    end;
  NStrInt:=s;
  end;

{ ---------------------------------------------------------------- }
(*$IFOPT N+ *)
(* Double-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrDbl (x   : double;
                  n,d : integer) : string;
var
  s : string;
  i : integer;
begin
  str (abs(x):n:d,s);
  if x<0 then if s[1]=Space then s[1]:='-' else s:='-'+s;
  for i:=1 to n do if s[i]=' ' then s[i]:='0';
  if DezTr<>Punkt then s:=ReplDecPt (s);
  ZStrDbl:=s;
  end;

{ ------------------------------------------------------------------ }
(* Double-Zahl in String in Dreiergruppen umsetzen*)
function NStrDbl (x : double;
                  n,d : integer) : string;
(* Format wie StrDbl *)
var
  s   : string;
  i,j : integer;
begin
  s:=StrDbl(x,n,d);
  i:=pos(DezTr,s);
  if i=0 then i:=succ(length(s));
  j:=i;
  while (j>4) do begin
    dec(j,3);
    if s[j-1]<>'-' then begin
      insert (ThMark,s,j); inc(i);
      end;
    end;
  inc(i,4);
  while (i<length(s)) do begin
    insert(ThMark,s,i); inc(i,4);
    end;
  NStrDbl:=s;
  end;

(*$ENDIF *)

{ ---------------------------------------------------------------- }
(* Konvertierung einer 16-bit oder 8-Bit Zahl in einen String,
   der dem Hex-Wert entspricht *)
function HexByte (b : byte) : string;
const
  HexTab : array [0..15] of char = '0123456789ABCDEF';
begin
  HexByte:=HexTab[b shr 4] + HexTab[b and $f];
  end;

function HexWord (w : word) : string;
begin
  HexWord:=HexByte(hi(w))+HexByte(lo(w));
  end;

{ ---------------------------------------------------------------- }
(*$IFOPT N+ *)
(* Wert auf Gltigkeit prfen
   true : x <> NoVal *)
function Valid (x,NoVal : double) : boolean;
begin
  Valid:=abs(x-NoVal)>0.1;
  end;
(*$ENDIF *)

{ ---------------------------------------------------------------- }
(* Dezimalpunkt durch vorgegeben Trenner (SetDecPtChar) ersetzen *)
function ReplDecPt (s : string) : string;
var
  i : byte;
begin
  i:=pos (Punkt,s);
  if i>0 then s[i]:=DezTr;
  ReplDecPt:=s;
  end;

(* aktuellen Dezimaltrenner durch Punkt ersetzen *)
function SetDecPt (s : string) : string;
var
  i : byte;
begin
  i:=pos (DezTr,s);
  if i>0 then s[i]:=Punkt;
  SetDecPt:=s;
  end;

{ ---------------------------------------------------------------- }
(* Longinteger in Hi- oder Lo-Wort zerlegen *)
function HiWord (n : longint) : word;
var
  Long : TLongWord absolute n;
begin
  HiWord:=Long.Hi;
  end;

function LoWord (n : longint) : word;
var
  Long : TLongWord absolute n;
begin
  LoWord:=Long.Lo;
  end;

(* Longinteger aus Hi- und Lo-Wort zusammensetzen *)
function JoinWord (HiWord,LoWord : word) : longint;
var
  Long : TLongWord;
begin
  with Long do begin
    Hi:=HiWord; Lo:=LoWord;
    JoinWord:=LongWord;
    end;
  end;

(* Longinteger aus 4 Bytes zusammensetzen *)
function JoinByte (HHByte,HLByte,LHByte,LLByte : word) : longint;
var
  Long : TLongWord;
begin
  with Long do begin
    LoL:=LLbyte; LoH:=LHByte;
    HiL:=HLByte; HiH:=HHByte;
    JoinByte:=LongWord;
    end;
  end;

end.
